#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Shreyas Srish (@shrsr) <ssrish@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: ndo_qos_dscp_cos_translation_policy
short_description: Manage QoS DSCP CoS Translation Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Quality of Service (QoS) Differentiated Services Code Point (DSCP) Class of Service (CoS) Translation Policies.
- This module can be used on Cisco Nexus Dashboard Orchestrator (NDO).
- There can only be a single QoS DSCP CoS translation policy in a fabric policy template.
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Shreyas Srish (@shrsr)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the fabric policy template.
    - This parameter or O(template) is required.
    type: str
  qos_dscp_translation_policy:
    description:
    - The name of the QoS DSCP Translation Policy.
    type: str
    aliases: [ name ]
  description:
    description:
    - The description of the QoS DSCP Translation Policy.
    type: str
  qos_dscp_translation_policy_uuid:
    description:
    - The UUID of the QoS DSCP Translation Policy.
    type: str
    aliases: [ uuid ]
  admin_state:
    description:
    - The administrative state of the policy.
    type: str
    choices: [ enabled, disabled ]
  control_plane_traffic:
    description:
    - DSCP value for control plane traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  policy_plane_traffic:
    description:
    - DSCP value for policy plane traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  span_traffic:
    description:
    - DSCP value for SPAN traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  traceroute_traffic:
    description:
    - DSCP value for traceroute traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  level1:
    description:
    - DSCP value for level 1 traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  level2:
    description:
    - DSCP value for level 2 traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  level3:
    description:
    - DSCP value for level 3 traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  level4:
    description:
    - DSCP value for level 4 traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  level5:
    description:
    - DSCP value for level 5 traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  level6:
    description:
    - DSCP value for level 6 traffic.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_template) module can be used for this.
- Attempts to create any additional QoS DSCP CoS translation policies will only update the existing
  object in the Fabric Policy template.
seealso:
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new QoS DSCP Translation policy
  cisco.mso.ndo_qos_dscp_cos_translation_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    qos_dscp_translation_policy: ansible_test_qos_dscp_policy
    admin_state: enabled
    control_plane_traffic: cs0
    policy_plane_traffic: cs4
    span_traffic: cs5
    traceroute_traffic: cs6
    level1: cs1
    level2: cs2
    level3: cs3
    level4: af11
    level5: af21
    level6: af31
    state: present
  register: create_qos_dscp_translation_policy

- name: Query QoS DSCP Translation policy with name
  cisco.mso.ndo_qos_dscp_cos_translation_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    qos_dscp_translation_policy: ansible_test_qos_dscp_policy
    state: query
  register: query_one

- name: Query QoS DSCP Translation policy with uuid
  cisco.mso.ndo_qos_dscp_cos_translation_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    qos_dscp_translation_policy_uuid: "{{ create_qos_dscp_translation_policy.current.uuid }}"
    state: query
  register: query_one_uuid

- name: Delete a QoS DSCP Translation policy with name
  cisco.mso.ndo_qos_dscp_cos_translation_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    qos_dscp_translation_policy: ansible_test_qos_dscp_policy
    state: absent

- name: Delete a QoS DSCP Translation policy with uuid
  cisco.mso.ndo_qos_dscp_cos_translation_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    qos_dscp_translation_policy_uuid: "{{ query_one.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""
import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
)
from ansible_collections.cisco.mso.plugins.module_utils.template import (
    MSOTemplate,
    KVPair,
)
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
from ansible_collections.cisco.mso.plugins.module_utils.constants import TARGET_DSCP_MAP


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        qos_dscp_translation_policy=dict(type="str", aliases=["name"]),
        description=dict(type="str"),
        qos_dscp_translation_policy_uuid=dict(type="str", aliases=["uuid"]),
        admin_state=dict(type="str", choices=["enabled", "disabled"]),
        control_plane_traffic=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        policy_plane_traffic=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        span_traffic=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        traceroute_traffic=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        level1=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        level2=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        level3=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        level4=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        level5=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        level6=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["qos_dscp_translation_policy", "qos_dscp_translation_policy_uuid"], True],
            ["state", "present", ["qos_dscp_translation_policy", "qos_dscp_translation_policy_uuid"], True],
        ],
        required_one_of=[
            ["template", "template_id"],
        ],
        mutually_exclusive=[
            ["template", "template_id"],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    qos_dscp_translation_policy = module.params.get("qos_dscp_translation_policy")
    description = module.params.get("description")
    qos_dscp_translation_policy_uuid = module.params.get("qos_dscp_translation_policy_uuid")
    admin_state = module.params.get("admin_state")
    control_plane_traffic = TARGET_DSCP_MAP.get(module.params.get("control_plane_traffic"))
    policy_plane_traffic = TARGET_DSCP_MAP.get(module.params.get("policy_plane_traffic"))
    span_traffic = TARGET_DSCP_MAP.get(module.params.get("span_traffic"))
    traceroute_traffic = TARGET_DSCP_MAP.get(module.params.get("traceroute_traffic"))
    level1 = TARGET_DSCP_MAP.get(module.params.get("level1"))
    level2 = TARGET_DSCP_MAP.get(module.params.get("level2"))
    level3 = TARGET_DSCP_MAP.get(module.params.get("level3"))
    level4 = TARGET_DSCP_MAP.get(module.params.get("level4"))
    level5 = TARGET_DSCP_MAP.get(module.params.get("level5"))
    level6 = TARGET_DSCP_MAP.get(module.params.get("level6"))
    state = module.params.get("state")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "fabric_policy", template, template_id)
    mso_template.validate_template("fabricPolicy")
    object_description = "QoS DSCP CoS Translation Policy"

    path = "/fabricPolicyTemplate/template/qosDscpTranslation"
    existing_qos_policies = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("qosDscpTranslation", {})
    if qos_dscp_translation_policy or qos_dscp_translation_policy_uuid:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            [existing_qos_policies],
            [KVPair("uuid", qos_dscp_translation_policy_uuid) if qos_dscp_translation_policy_uuid else KVPair("name", qos_dscp_translation_policy)],
        )
        if match:
            mso.existing = mso.previous = copy.deepcopy(match.details)
    else:
        mso.existing = mso.previous = existing_qos_policies

    if state == "present":
        mso_values = dict(
            name=qos_dscp_translation_policy,
            description=description,
            adminState=admin_state,
            controlPlaneTraffic=control_plane_traffic,
            policyPlaneTraffic=policy_plane_traffic,
            spanTraffic=span_traffic,
            tracerouteTraffic=traceroute_traffic,
            level1=level1,
            level2=level2,
            level3=level3,
            level4=level4,
            level5=level5,
            level6=level6,
        )
        if match:
            append_update_ops_data(ops, match.details, path, mso_values)
            mso.sanitize(match.details, collate=True)

        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        qos_policies = response.get("fabricPolicyTemplate", {}).get("template", {}).get("qosDscpTranslation", {})
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            [qos_policies],
            [KVPair("uuid", qos_dscp_translation_policy_uuid) if qos_dscp_translation_policy_uuid else KVPair("name", qos_dscp_translation_policy)],
        )
        if match:
            mso.existing = match.details
        else:
            mso.existing = {}
    elif module.check_mode and state != "query":
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, Evgeniy Krysanov <evgeniy.krysanov@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: bitbucket_pipeline_key_pair
short_description: Manages Bitbucket pipeline SSH key pair
description:
  - Manages Bitbucket pipeline SSH key pair.
author:
  - Evgeniy Krysanov (@catcombo)
extends_documentation_fragment:
  - community.general._bitbucket
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  repository:
    description:
      - The repository name.
    type: str
    required: true
  workspace:
    description:
      - The repository owner.
      - B(Note:) O(ignore:username) used to be an alias of this option. Since community.general 6.0.0 it is an alias of O(user).
    type: str
    required: true
  public_key:
    description:
      - The public key.
    type: str
  private_key:
    description:
      - The private key.
    type: str
  state:
    description:
      - Indicates desired state of the key pair.
    type: str
    required: true
    choices: [absent, present]
notes:
  - Check mode is supported.
"""

EXAMPLES = r"""
- name: Create or update SSH key pair
  community.general.bitbucket_pipeline_key_pair:
    repository: 'bitbucket-repo'
    workspace: bitbucket_workspace
    public_key: '{{lookup("file", "bitbucket.pub") }}'
    private_key: '{{lookup("file", "bitbucket") }}'
    state: present

- name: Remove SSH key pair
  community.general.bitbucket_pipeline_key_pair:
    repository: bitbucket-repo
    workspace: bitbucket_workspace
    state: absent
"""

RETURN = r""" # """

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._bitbucket import BitbucketHelper

error_messages = {
    "invalid_params": "Account, repository or SSH key pair was not found",
    "required_keys": "`public_key` and `private_key` are required when the `state` is `present`",
}

BITBUCKET_API_ENDPOINTS = {
    "ssh-key-pair": f"{BitbucketHelper.BITBUCKET_API_URL}/2.0/repositories/{{workspace}}/{{repo_slug}}/pipelines_config/ssh/key_pair",
}


def get_existing_ssh_key_pair(module, bitbucket):
    """
    Retrieves an existing ssh key pair from repository
    specified in module param `repository`

    :param module: instance of the :class:`AnsibleModule`
    :param bitbucket: instance of the :class:`BitbucketHelper`
    :return: existing key pair or None if not found
    :rtype: dict or None

    Return example::

        {
            "public_key": "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQ...2E8HAeT",
            "type": "pipeline_ssh_key_pair"
        }
    """
    api_url = BITBUCKET_API_ENDPOINTS["ssh-key-pair"].format(
        workspace=module.params["workspace"],
        repo_slug=module.params["repository"],
    )

    info, content = bitbucket.request(
        api_url=api_url,
        method="GET",
    )

    if info["status"] == 404:
        # Account, repository or SSH key pair was not found.
        return None

    return content


def update_ssh_key_pair(module, bitbucket):
    info, content = bitbucket.request(
        api_url=BITBUCKET_API_ENDPOINTS["ssh-key-pair"].format(
            workspace=module.params["workspace"],
            repo_slug=module.params["repository"],
        ),
        method="PUT",
        data={
            "private_key": module.params["private_key"],
            "public_key": module.params["public_key"],
        },
    )

    if info["status"] == 404:
        module.fail_json(msg=error_messages["invalid_params"])

    if info["status"] != 200:
        module.fail_json(msg=f"Failed to create or update pipeline ssh key pair : {info}")


def delete_ssh_key_pair(module, bitbucket):
    info, content = bitbucket.request(
        api_url=BITBUCKET_API_ENDPOINTS["ssh-key-pair"].format(
            workspace=module.params["workspace"],
            repo_slug=module.params["repository"],
        ),
        method="DELETE",
    )

    if info["status"] == 404:
        module.fail_json(msg=error_messages["invalid_params"])

    if info["status"] != 204:
        module.fail_json(msg=f"Failed to delete pipeline ssh key pair: {info}")


def main():
    argument_spec = BitbucketHelper.bitbucket_argument_spec()
    argument_spec.update(
        repository=dict(type="str", required=True),
        workspace=dict(type="str", required=True),
        public_key=dict(type="str"),
        private_key=dict(type="str", no_log=True),
        state=dict(type="str", choices=["present", "absent"], required=True),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=BitbucketHelper.bitbucket_required_one_of(),
        required_together=BitbucketHelper.bitbucket_required_together(),
    )

    bitbucket = BitbucketHelper(module)

    state = module.params["state"]
    public_key = module.params["public_key"]
    private_key = module.params["private_key"]

    # Check parameters
    if ((public_key is None) or (private_key is None)) and (state == "present"):
        module.fail_json(msg=error_messages["required_keys"])

    # Retrieve access token for authorized API requests
    bitbucket.fetch_access_token()

    # Retrieve existing ssh key
    key_pair = get_existing_ssh_key_pair(module, bitbucket)
    changed = False

    # Create or update key pair
    if (not key_pair or (key_pair.get("public_key") != public_key)) and (state == "present"):
        if not module.check_mode:
            update_ssh_key_pair(module, bitbucket)
        changed = True

    # Delete key pair
    elif key_pair and (state == "absent"):
        if not module.check_mode:
            delete_ssh_key_pair(module, bitbucket)
        changed = True

    module.exit_json(changed=changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: ucs_vlan_find
short_description: Find VLANs on Cisco UCS Manager
description:
- Find VLANs on Cisco UCS Manager based on different criteria.
extends_documentation_fragment: cisco.ucs.ucs
options:
  pattern:
    description:
    - Regex pattern to find within the name property of the fabricVlan class.
    - This is required if C(vlanid) parameter is not supplied.
    type: str
  fabric:
    description:
    - "The fabric configuration of the VLAN.  This can be one of the following:"
    - "common - The VLAN applies to both fabrics and uses the same configuration parameters in both cases."
    - "A — The VLAN only applies to fabric A."
    - "B — The VLAN only applies to fabric B."
    choices: [common, A, B]
    default: common
    type: str
  vlanid:
    description:
    - The unique string identifier assigned to the VLAN.
    - A VLAN ID can be between '1' and '3967', or between '4048' and '4093'.
    - This is required if C(pattern) parameter is not supplied.
    type: str
requirements:
- ucsmsdk
author:
- David Martinez (@dx0xm)
- David Soper (@dsoper2)
- John McDonough (@movinalot)
- CiscoUcs (@CiscoUcs)
'''

EXAMPLES = r'''
- name: Get all vlans in fabric A
  cisco.ucs.ucs_vlan_find:
    hostname: 172.16.143.150
    username: admin
    password: password
    fabric: 'A'
    pattern: '.'
- name: Confirm if vlan 15 is present
  cisco.ucs.ucs_vlan_find:
    hostname: 172.16.143.150
    username: admin
    password: password
    vlanid: '15'
'''

RETURN = r'''
vlan_list:
    description: basic details of vlans found
    returned: on success
    type: list
    sample: [
        {
            "id": "0",
            "name": "vlcloud1"
        }
    ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ucs.plugins.module_utils.ucs import UCSModule, ucs_argument_spec


def main():
    argument_spec = ucs_argument_spec.copy()
    argument_spec.update(
        fabric=dict(type='str', default='common', choices=['common', 'A', 'B']),
        pattern=dict(type='str'),
        vlanid=dict(type='str')
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=[['pattern', 'vlanid']]
    )

    ucs = UCSModule(module)

    filtls = ['(cloud,"ethlan")']
    if module.params['fabric'] != 'common':
        filtls.append('(switch_id,"' + module.params['fabric'] + '")')
    if module.params['vlanid']:
        filtls.append('(id,"' + module.params['vlanid'] + '")')
    else:
        filtls.append('(name,"' + module.params['pattern'] + '")')

    object_dict = ucs.login_handle.query_classid("fabricVlan", filter_str=' and '.join(filtls))

    if object_dict is None:
        module.fail_json(msg="Failed to query vlan objects")

    vlnlist = []
    for ob in object_dict:
        vlnlist.append(dict(name=ob.name, id=ob.id))

    module.exit_json(changed=False,
                     vlan_list=vlnlist)


if __name__ == '__main__':
    main()



# (c) 2015, Joerg Thalheim <joerg@higgsboson.tk>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Joerg Thalheim (!UNKNOWN) <joerg@higgsboson.tk>
name: lxc
short_description: Run tasks in LXC containers using lxc python library
description:
  - Run commands or put/fetch files to an existing LXC container using lxc python library.
options:
  remote_addr:
    description:
      - Container identifier.
    type: string
    default: inventory_hostname
    vars:
      - name: inventory_hostname
      - name: ansible_host
      - name: ansible_lxc_host
  executable:
    default: /bin/sh
    description:
      - Shell executable.
    type: string
    vars:
      - name: ansible_executable
      - name: ansible_lxc_executable
"""

import errno
import fcntl
import os
import select
import shutil
import traceback

HAS_LIBLXC = False
try:
    import lxc as _lxc

    HAS_LIBLXC = True
except ImportError:
    pass

from ansible import errors
from ansible.module_utils.common.text.converters import to_bytes, to_native
from ansible.plugins.connection import ConnectionBase


class Connection(ConnectionBase):
    """Local lxc based connections"""

    transport = "community.general.lxc"
    has_pipelining = True
    default_user = "root"

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super().__init__(play_context, new_stdin, *args, **kwargs)

        self.container_name = None
        self.container = None

    def _connect(self):
        """connect to the lxc; nothing to do here"""
        super()._connect()

        if not HAS_LIBLXC:
            msg = "lxc python bindings are not installed"
            raise errors.AnsibleError(msg)

        container_name = self.get_option("remote_addr")
        if self.container and self.container_name == container_name:
            return

        self.container_name = container_name

        self._display.vvv("THIS IS A LOCAL LXC DIR", host=self.container_name)
        self.container = _lxc.Container(self.container_name)
        if self.container.state == "STOPPED":
            raise errors.AnsibleError(f"{self.container_name} is not running")

    @staticmethod
    def _communicate(pid, in_data, stdin, stdout, stderr):
        buf = {stdout: [], stderr: []}
        read_fds = [stdout, stderr]
        if in_data:
            write_fds = [stdin]
        else:
            write_fds = []
        while len(read_fds) > 0 or len(write_fds) > 0:
            try:
                ready_reads, ready_writes, dummy = select.select(read_fds, write_fds, [])
            except OSError as e:
                if e.args[0] == errno.EINTR:
                    continue
                raise
            for fd in ready_writes:
                in_data = in_data[os.write(fd, in_data) :]
                if len(in_data) == 0:
                    write_fds.remove(fd)
            for fd in ready_reads:
                data = os.read(fd, 32768)
                if not data:
                    read_fds.remove(fd)
                buf[fd].append(data)

        (pid, returncode) = os.waitpid(pid, 0)

        return returncode, b"".join(buf[stdout]), b"".join(buf[stderr])

    def _set_nonblocking(self, fd):
        flags = fcntl.fcntl(fd, fcntl.F_GETFL) | os.O_NONBLOCK
        fcntl.fcntl(fd, fcntl.F_SETFL, flags)
        return fd

    def exec_command(self, cmd, in_data=None, sudoable=False):
        """run a command on the chroot"""
        super().exec_command(cmd, in_data=in_data, sudoable=sudoable)

        # python2-lxc needs bytes. python3-lxc needs text.
        executable = to_native(self.get_option("executable"), errors="surrogate_or_strict")
        local_cmd = [executable, "-c", to_native(cmd, errors="surrogate_or_strict")]

        read_stdout, write_stdout = None, None
        read_stderr, write_stderr = None, None
        read_stdin, write_stdin = None, None

        try:
            read_stdout, write_stdout = os.pipe()
            read_stderr, write_stderr = os.pipe()

            kwargs = {
                "stdout": self._set_nonblocking(write_stdout),
                "stderr": self._set_nonblocking(write_stderr),
                "env_policy": _lxc.LXC_ATTACH_CLEAR_ENV,
            }

            if in_data:
                read_stdin, write_stdin = os.pipe()
                kwargs["stdin"] = self._set_nonblocking(read_stdin)

            self._display.vvv(f"EXEC {local_cmd}", host=self.container_name)
            pid = self.container.attach(_lxc.attach_run_command, local_cmd, **kwargs)
            if pid == -1:
                msg = f"failed to attach to container {self.container_name}"
                raise errors.AnsibleError(msg)

            write_stdout = os.close(write_stdout)
            write_stderr = os.close(write_stderr)
            if read_stdin:
                read_stdin = os.close(read_stdin)

            return self._communicate(pid, in_data, write_stdin, read_stdout, read_stderr)
        finally:
            fds = [read_stdout, write_stdout, read_stderr, write_stderr, read_stdin, write_stdin]
            for fd in fds:
                if fd:
                    os.close(fd)

    def put_file(self, in_path, out_path):
        """transfer a file from local to lxc"""
        super().put_file(in_path, out_path)
        self._display.vvv(f"PUT {in_path} TO {out_path}", host=self.container_name)
        in_path = to_bytes(in_path, errors="surrogate_or_strict")
        out_path = to_bytes(out_path, errors="surrogate_or_strict")

        if not os.path.exists(in_path):
            msg = f"file or module does not exist: {in_path}"
            raise errors.AnsibleFileNotFound(msg)
        try:
            src_file = open(in_path, "rb")
        except OSError as e:
            traceback.print_exc()
            raise errors.AnsibleError(f"failed to open input file to {in_path}") from e
        try:

            def write_file(args):
                with open(out_path, "wb+") as dst_file:
                    shutil.copyfileobj(src_file, dst_file)

            try:
                self.container.attach_wait(write_file, None)
            except OSError as e:
                traceback.print_exc()
                msg = f"failed to transfer file to {out_path}"
                raise errors.AnsibleError(msg) from e
        finally:
            src_file.close()

    def fetch_file(self, in_path, out_path):
        """fetch a file from lxc to local"""
        super().fetch_file(in_path, out_path)
        self._display.vvv(f"FETCH {in_path} TO {out_path}", host=self.container_name)
        in_path = to_bytes(in_path, errors="surrogate_or_strict")
        out_path = to_bytes(out_path, errors="surrogate_or_strict")

        try:
            dst_file = open(out_path, "wb")
        except OSError as e:
            traceback.print_exc()
            msg = f"failed to open output file {out_path}"
            raise errors.AnsibleError(msg) from e
        try:

            def write_file(args):
                try:
                    with open(in_path, "rb") as src_file:
                        shutil.copyfileobj(src_file, dst_file)
                finally:
                    # this is needed in the lxc child process
                    # to flush internal python buffers
                    dst_file.close()

            try:
                self.container.attach_wait(write_file, None)
            except OSError as e:
                traceback.print_exc()
                msg = f"failed to transfer file from {in_path} to {out_path}"
                raise errors.AnsibleError(msg) from e
        finally:
            dst_file.close()

    def close(self):
        """terminate the connection; nothing to do here"""
        super().close()
        self._connected = False



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_power_policy
short_description: Power Policy configuration for Cisco Intersight
description:
  - Manages Power Policy configuration on Cisco Intersight.
  - A policy to configure the power settings on Cisco Intersight managed servers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/power/Policy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Power Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Power Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  target_platform:
    description:
      - The platform type for which the power policy is intended. This determines which settings are applicable.
      - This parameter is required when C(state) is C(present).
    type: str
    choices: ['All', 'Standalone', 'FIAttached', 'Chassis']
  power_profiling:
    description:
      - Sets the Power Profiling of the Server.
      - If Enabled, this field allows the power manager to run power profiling utility to determine the power needs of the server.
      - This field is only supported for Cisco UCS X series servers.
      - Applicable for 'FIAttached' and 'All' platforms.
    type: str
    choices: ['Enabled', 'Disabled']
    default: 'Enabled'
  power_priority:
    description:
      - Sets the Power Priority of the Server. This priority is used to determine the initial power allocation for servers.
      - This field is only supported for Cisco UCS B series and X series servers.
      - Applicable for 'FIAttached' and 'All' platforms.
    type: str
    choices: ['Low', 'Medium', 'High']
    default: 'Low'
  power_restore:
    description:
      - Sets the Power Restore State of the Server.
      - In the absence of Intersight connectivity, the chassis/server will use this policy to recover the host power after a power loss event.
      - Applicable for 'Standalone', 'FIAttached', and 'All' platforms.
    type: str
    choices: ['AlwaysOff', 'AlwaysOn', 'LastState']
    default: 'AlwaysOff'
  power_redundancy:
    description:
      - Sets the Power Redundancy Mode of the Chassis. Redundancy Mode determines the number of PSUs the chassis keeps as redundant.
      - N+2 mode is only supported for Cisco UCS X series Chassis.
      - Applicable for 'Chassis' and 'All' platforms.
    type: str
    choices: ['Grid', 'NotRedundant', 'N+1', 'N+2']
    default: 'Grid'
  processor_package_power_limit:
    description:
      - Sets the Processor Package Power Limit (PPL) of a server. PPL refers to the amount of power that a CPU can draw from the power supply.
      - The Processor Package Power Limit (PPL) feature is currently available exclusively on Cisco UCS C225/C245 M8 servers.
      - Applicable for 'Standalone', 'FIAttached', and 'All' platforms.
    type: str
    choices: ['Default', 'Maximum', 'Minimum']
    default: 'Default'
  power_save_mode:
    description:
      - Sets the power save mode of the chassis.
      - If the requested power budget is less than available power capacity,
        the additional PSUs not required to comply with redundancy policy are placed in power save mode.
      - Applicable for 'Chassis' and 'All' platforms.
    type: str
    choices: ['Enabled', 'Disabled']
    default: 'Enabled'
  dynamic_power_rebalancing:
    description:
      - Sets the dynamic power rebalancing mode of the chassis.
      - If enabled, this mode allows the chassis to dynamically reallocate the power between servers depending on their power usage.
      - Applicable for 'Chassis' and 'All' platforms.
    type: str
    choices: ['Enabled', 'Disabled']
    default: 'Enabled'
  extended_power_capacity:
    description:
      - Sets the Extended Power Capacity of the Chassis.
      - If Enabled, this mode allows chassis available power to be increased by borrowing power from redundant power supplies.
      - This option is only supported for Cisco UCS X series Chassis.
      - Applicable for 'Chassis' and 'All' platforms.
    type: str
    choices: ['Enabled', 'Disabled']
    default: 'Enabled'
  power_allocation:
    description:
      - Sets the limit for the maximum input power consumption by the chassis (in Watts). Set to 0 for no limit.
      - Applicable for 'Chassis' and 'All' platforms.
    type: int
    default: 0
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a Power Policy for a Standalone Server
  cisco.intersight.intersight_power_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Standalone-Server-Power-Policy"
    description: "Power policy for standalone servers, restores to last state."
    target_platform: "Standalone"
    power_restore: "LastState"
    processor_package_power_limit: "Minimum"
    tags:
      - Key: "Owner"
        Value: "DevOps"

- name: Create a Power Policy for a Chassis with N+1 Redundancy
  cisco.intersight.intersight_power_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Chassis-N1-Redundancy"
    description: "Power policy for chassis with N+1 redundancy."
    target_platform: "Chassis"
    power_redundancy: "N+1"
    power_save_mode: "Disabled"
    power_allocation: 7500

- name: Create a universal Power Policy for FIAttached Servers
  cisco.intersight.intersight_power_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "FI-Attached-Default"
    description: "Default power policy for all FIAttached servers."
    target_platform: "FIAttached"
    power_profiling: "Enabled"
    power_priority: "Medium"

- name: Delete a Power Policy
  cisco.intersight.intersight_power_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Standalone-Server-Power-Policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "FI-Attached-Default",
        "ObjectType": "power.Policy",
        "PowerPriority": "Medium",
        "PowerProfiling": "Enabled",
        "PowerRestoreState": "AlwaysOff",
        "Tags": [],
        ...
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec

ALL = 'All'
STANDALONE_SERVER = 'Standalone'
FI_ATTACHED_SERVER = 'FIAttached'
CHASSIS = 'Chassis'


def validate_input(module: AnsibleModule):
    power_allocation = module.params['power_allocation']
    if 0 > power_allocation or power_allocation > 65535:
        module.fail_json(msg=f"power_allocation has to be between 0 and 65535, current value is: {power_allocation}")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        target_platform=dict(
            type='str',
            choices=[ALL, STANDALONE_SERVER, FI_ATTACHED_SERVER, CHASSIS]
        ),
        power_profiling=dict(
            type='str',
            choices=['Enabled', 'Disabled'],
            default='Enabled'
        ),
        power_priority=dict(
            type='str',
            choices=['Low', 'Medium', 'High'],
            default='Low'
        ),
        power_restore=dict(
            type='str',
            choices=['AlwaysOff', 'AlwaysOn', 'LastState'],
            default='AlwaysOff'
        ),
        power_redundancy=dict(
            type='str',
            choices=['Grid', 'NotRedundant', 'N+1', 'N+2'],
            default='Grid'
        ),
        processor_package_power_limit=dict(
            type='str',
            choices=['Default', 'Maximum', 'Minimum'],
            default='Default'
        ),
        power_save_mode=dict(
            type='str',
            choices=['Enabled', 'Disabled'],
            default='Enabled'
        ),
        dynamic_power_rebalancing=dict(
            type='str',
            choices=['Enabled', 'Disabled'],
            default='Enabled'
        ),
        extended_power_capacity=dict(
            type='str',
            choices=['Enabled', 'Disabled'],
            default='Enabled'
        ),
        power_allocation=dict(
            type='int',
            default=0
        ),
    )
    required_if = [
        ('state', 'present', ['target_platform']),
    ]
    module = AnsibleModule(
        argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    validate_input(module)
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/power/Policies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        platform = module.params['target_platform']

        # Chassis-specific parameters
        if platform in [CHASSIS, ALL]:
            intersight.api_body.update({
                "RedundancyMode": module.params['power_redundancy'],
                "PowerSaveMode": module.params['power_save_mode'],
                "DynamicRebalancing": module.params['dynamic_power_rebalancing'],
                "ExtendedPowerCapacity": module.params['extended_power_capacity'],
                "AllocatedBudget": module.params['power_allocation']
            })

        # Server-specific parameters (Standalone and FIAttached)
        if platform in [STANDALONE_SERVER, FI_ATTACHED_SERVER, ALL]:
            intersight.api_body.update({
                "ProcessorPackagePowerLimit": module.params['processor_package_power_limit'],
                "PowerRestoreState": module.params['power_restore']
            })

        # FIAttached specific parameters
        if platform in [FI_ATTACHED_SERVER, ALL]:
            intersight.api_body.update({
                "PowerPriority": module.params['power_priority'],
                "PowerProfiling": module.params['power_profiling']
            })

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
author:
  - Kevin Breit (@kbreit)
deprecated:
  alternative: cisco.meraki.organizations_appliance_vpn_third_party_vpnpeers
  removed_in: 3.0.0
  why: Updated modules released with increased functionality
description:
  - Create, edit, query, or delete third party VPN peers in a Meraki environment.
extends_documentation_fragment: cisco.meraki.meraki
module: meraki_mx_third_party_vpn_peers
options:
  peers:
    description:
      - The list of VPN peers.
    elements: dict
    suboptions:
      ike_version:
        choices:
          - '1'
          - '2'
        default: '1'
        description:
          - The IKE version to be used for the IPsec VPN peer configuration.
        type: str
      ipsec_policies:
        description:
          - Custom IPSec policies for the VPN peer. If not included and a preset has
            not been chosen, the default preset for IPSec policies will be used.
        suboptions:
          child_auth_algo:
            choices:
              - sha256
              - sha1
              - md5
            description:
              - This is the authentication algorithms to be used in Phase 2.
            elements: str
            type: list
          child_cipher_algo:
            choices:
              - aes256
              - aes192
              - aes128
              - tripledes
              - des
              - 'null'
            description:
              - This is the cipher algorithms to be used in Phase 2.
            elements: str
            type: list
          child_lifetime:
            description:
              - The lifetime of the Phase 2 SA in seconds.
            type: int
          child_pfs_group:
            choices:
              - disabled
              - group14
              - group5
              - group2
              - group1
            description:
              - This is the Diffie-Hellman group to be used for Perfect Forward Secrecy
                in Phase 2.
            elements: str
            type: list
          ike_auth_algo:
            choices:
              - sha256
              - sha1
              - md5
            description:
              - This is the authentication algorithm to be used in Phase 1.
            elements: str
            type: list
          ike_cipher_algo:
            choices:
              - aes256
              - aes192
              - aes128
              - tripledes
              - des
            description:
              - This is the cipher algorithm to be used in Phase 1.
            elements: str
            type: list
          ike_diffie_hellman_group:
            choices:
              - group14
              - group5
              - group2
              - group1
            description:
              - This is the Diffie-Hellman group to be used in Phase 1.
            elements: str
            type: list
          ike_lifetime:
            description:
              - The lifetime of the Phase 1 SA in seconds.
            type: int
          ike_prf_algo:
            choices:
              - prfsha256
              - prfsha1
              - prfmd5
              - default
            description:
              - This is the pseudo-random function to be used in IKE_SA.
            elements: str
            type: list
        type: dict
      ipsec_policies_preset:
        choices:
          - default
          - aws
          - azure
        description:
          - Specifies IPsec preset values. If this is provided, the 'ipsecPolicies'
            parameter is ignored.
        type: str
      name:
        description:
          - The name of the VPN peer.
          - Required when state is present.
        type: str
      network_tags:
        description:
          - A list of network tags that will connect with this peer. If not included,
            the default is ['all'].
        elements: str
        type: list
      private_subnets:
        description:
          - The list of the private subnets of the VPN peer.
          - Required when state is present.
        elements: str
        type: list
      public_ip:
        description:
          - The public IP of the VPN peer.
          - Required when state is present.
        type: str
      remote_id:
        description:
          - The remote ID is used to identify the connecting VPN peer. This can either
            be a valid IPv4 Address, FQDN or User FQDN.
        type: str
      secret:
        description:
          - The shared secret with the VPN peer.
          - Required when state is present.
        type: str
    type: list
  state:
    choices:
      - absent
      - present
      - query
    default: query
    description:
      - Specifies whether object should be queried, created/modified, or removed.
    type: str
short_description: Manage third party (IPSec) VPN peers for MX devices
"""

EXAMPLES = r"""
- name: Query all VPN peers
  meraki_mx_third_party_vpn_peers:
    auth_key: abc123
    state: query
    org_name: orgName
- name: Create VPN peer with an IPsec policy
  meraki_mx_third_party_vpn_peers:
    auth_key: abc123
    state: present
    org_name: orgName
    peers:
      - name: Test peer
        public_ip: 198.51.100.1
        secret: s3cret
        private_subnets:
          - 192.0.2.0/24
        ike_version: '2'
        network_tags:
          - none
        remote_id: 192.0.2.0
        ipsec_policies:
          child_lifetime: 600
          ike_lifetime: 600
          child_auth_algo:
            - md5
          child_cipher_algo:
            - tripledes
            - aes192
          child_pfs_group:
            - disabled
          ike_auth_algo:
            - sha256
          ike_cipher_algo:
            - tripledes
          ike_diffie_hellman_group:
            - group2
          ike_prf_algo:
            - prfmd5
"""

RETURN = r"""
response:
  description: Information about the organization which was created or modified
  returned: success
  type: complex
  contains:
    appliance_ip:
      description: IP address of Meraki appliance in the VLAN
      returned: success
      type: str
      sample: 192.0.1.1
    dnsnamservers:
      description: IP address or Meraki defined DNS servers which VLAN should use by default
      returned: success
      type: str
      sample: upstream_dns
    peers:
      description: The list of VPN peers.
      returned: success
      type: complex
      contains:
        ike_version:
          description: The IKE version to be used for the IPsec VPN peer configuration.
          returned: success
          type: str
          sample: "1"
        ipsec_policies_preset:
          description: Preconfigured IPsec settings.
          returned: success
          type: str
          sample: "aws"
        name:
          description: The name of the VPN peer.
          returned: success
          type: str
          sample: "MyVPNPeer"
        public_ip:
          description: The public IP of the VPN peer.
          returned: success
          type: str
          sample: "198.51.100.1"
        remote_id:
          description: "The remote ID is used to identify the connecting VPN peer."
          returned: success
          type: str
          sample: "s3cret"
        network_tags:
          description: A list of network tags that will connect with this peer.
          returned: success
          type: list
          sample: ["all"]
        private_subnets:
          description: The list of the private subnets of the VPN peer.
          returned: success
          type: list
          sample: ["192.0.2.0/24"]
        ipsec_policies:
          description: Custom IPSec policies for the VPN peer.
          returned: success
          type: complex
          contains:
            child_lifetime:
              description: The lifetime of the Phase 2 SA in seconds.
              returned: success
              type: str
              sample: "60"
            ike_lifetime:
              description: The lifetime of the Phase 1 SA in seconds.
              returned: success
              type: str
              sample: "60"
            child_auth_algo:
              description: This is the authentication algorithms to be used in Phase 2.
              returned: success
              type: list
              sample: ["sha1"]
            child_cipher_algo:
              description: This is the cipher algorithms to be used in Phase 2.
              returned: success
              type: list
              sample: ["aes192"]
            child_pfs_group:
              description: This is the Diffie-Hellman group to be used for Perfect Forward Secrecy in Phase 2.
              returned: success
              type: list
              sample: ["group14"]
            ike_auth_algo:
              description: This is the authentication algorithm to be used in Phase 1.
              returned: success
              type: list
              sample: ["sha1"]
            ike_cipher_algo:
              description: This is the cipher algorithm to be used in Phase 1.
              returned: success
              type: list
              sample: ["aes128"]
            ike_diffie_hellman_group:
              description: This is the Diffie-Hellman group to be used in Phase 1.
              returned: success
              type: list
              sample: ["group14"]
            ike_prf_algo:
              description: This is the pseudo-random function to be used in IKE_SA.
              returned: success
              type: list
              sample: ["prfmd5"]
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)
import json


def validate_payload(meraki):
    for peer in meraki.params["peers"]:
        if peer["name"] is None:
            meraki.fail_json(msg="Peer name must be specified")
        elif peer["public_ip"] is None:
            meraki.fail_json(msg="Peer public IP must be specified")
        elif peer["secret"] is None:
            meraki.fail_json(msg="Peer secret must be specified")
        elif peer["private_subnets"] is None:
            meraki.fail_json(msg="Peer private subnets must be specified")


def construct_payload(meraki):
    validate_payload(meraki)
    peer_list = []
    for peer in meraki.params["peers"]:
        current_peer = dict()
        current_peer["name"] = peer["name"]
        current_peer["publicIp"] = peer["public_ip"]
        current_peer["secret"] = peer["secret"]
        current_peer["privateSubnets"] = peer["private_subnets"]
        if peer["ike_version"] is not None:
            current_peer["ikeVersion"] = peer["ike_version"]
        if peer["ipsec_policies_preset"] is not None:
            current_peer["ipsecPoliciesPreset"] = peer["ipsec_policies_preset"]
        if peer["remote_id"] is not None:
            current_peer["remoteId"] = peer["remote_id"]
        if peer["network_tags"] is not None:
            current_peer["networkTags"] = peer["network_tags"]
        if peer["ipsec_policies"] is not None:
            current_peer["ipsecPolicies"] = dict()
            if peer["ipsec_policies"]["child_lifetime"] is not None:
                current_peer["ipsecPolicies"]["childLifetime"] = peer["ipsec_policies"][
                    "child_lifetime"
                ]
            if peer["ipsec_policies"]["ike_lifetime"] is not None:
                current_peer["ipsecPolicies"]["ikeLifetime"] = peer["ipsec_policies"][
                    "ike_lifetime"
                ]
            if peer["ipsec_policies"]["child_auth_algo"] is not None:
                current_peer["ipsecPolicies"]["childAuthAlgo"] = peer["ipsec_policies"][
                    "child_auth_algo"
                ]
            if peer["ipsec_policies"]["child_cipher_algo"] is not None:
                current_peer["ipsecPolicies"]["childCipherAlgo"] = peer[
                    "ipsec_policies"
                ]["child_cipher_algo"]
            if peer["ipsec_policies"]["child_pfs_group"] is not None:
                current_peer["ipsecPolicies"]["childPfsGroup"] = peer["ipsec_policies"][
                    "child_pfs_group"
                ]
            if peer["ipsec_policies"]["ike_auth_algo"] is not None:
                current_peer["ipsecPolicies"]["ikeAuthAlgo"] = peer["ipsec_policies"][
                    "ike_auth_algo"
                ]
            if peer["ipsec_policies"]["ike_cipher_algo"] is not None:
                current_peer["ipsecPolicies"]["ikeCipherAlgo"] = peer["ipsec_policies"][
                    "ike_cipher_algo"
                ]
            if peer["ipsec_policies"]["ike_diffie_hellman_group"] is not None:
                current_peer["ipsecPolicies"]["ikeDiffieHellmanGroup"] = peer[
                    "ipsec_policies"
                ]["ike_diffie_hellman_group"]
            if peer["ipsec_policies"]["ike_prf_algo"] is not None:
                current_peer["ipsecPolicies"]["ikePrfAlgo"] = peer["ipsec_policies"][
                    "ike_prf_algo"
                ]

        peer_list.append(current_peer)
    payload = {"peers": peer_list}
    return payload


def main():
    # define the available arguments/parameters that a user can pass to
    # the module

    ipsec_policies_arg_spec = dict(
        child_lifetime=dict(type="int", default=None),
        ike_lifetime=dict(type="int", default=None),
        child_auth_algo=dict(
            type="list", elements="str", default=None, choices=["sha256", "sha1", "md5"]
        ),
        child_cipher_algo=dict(
            type="list",
            elements="str",
            default=None,
            choices=["aes256", "aes192", "aes128", "tripledes", "des", "null"],
        ),
        child_pfs_group=dict(
            type="list",
            elements="str",
            default=None,
            choices=["disabled", "group14", "group5", "group2", "group1"],
        ),
        ike_auth_algo=dict(
            type="list", elements="str", default=None, choices=["sha256", "sha1", "md5"]
        ),
        ike_cipher_algo=dict(
            type="list",
            elements="str",
            default=None,
            choices=["aes256", "aes192", "aes128", "tripledes", "des"],
        ),
        ike_diffie_hellman_group=dict(
            type="list",
            elements="str",
            default=None,
            choices=["group14", "group5", "group2", "group1"],
        ),
        ike_prf_algo=dict(
            type="list",
            elements="str",
            default=None,
            choices=["prfsha256", "prfsha1", "prfmd5", "default"],
        ),
    )

    peers_arg_spec = dict(
        name=dict(type="str"),
        public_ip=dict(type="str"),
        secret=dict(type="str", no_log=True),
        private_subnets=dict(type="list", elements="str"),
        ike_version=dict(type="str", choices=["1", "2"], default="1"),
        ipsec_policies_preset=dict(
            type="str", choices=["default", "aws", "azure"], default=None
        ),
        remote_id=dict(type="str", default=None),
        network_tags=dict(type="list", elements="str", default=None),
        ipsec_policies=dict(type="dict", options=ipsec_policies_arg_spec, default=None),
    )

    argument_spec = meraki_argument_spec()
    argument_spec.update(
        state=dict(type="str", choices=["absent", "present", "query"], default="query"),
        peers=dict(type="list", elements="dict", options=peers_arg_spec),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    meraki = MerakiModule(module, function="third_party_vpn_peer")

    meraki.params["follow_redirects"] = "all"

    query_urls = {
        "third_party_vpn_peer": "/organizations/{org_id}/appliance/vpn/thirdPartyVPNPeers"
    }
    update_url = {
        "third_party_vpn_peer": "/organizations/{org_id}/appliance/vpn/thirdPartyVPNPeers"
    }

    meraki.url_catalog["get_all"].update(query_urls)
    meraki.url_catalog["update"] = update_url

    payload = None
    if meraki.params["org_id"] is None and meraki.params["org_name"] is None:
        meraki.fail_json(msg="Organization must be specified via org_name or org_id")

    org_id = meraki.params["org_id"]
    if org_id is None:
        org_id = meraki.get_org_id(meraki.params["org_name"])

    if meraki.params["state"] == "query":
        path = meraki.construct_path("get_all", org_id=org_id)
        response = meraki.request(path, "GET")
        meraki.result["data"] = response
    elif meraki.params["state"] == "present":
        payload = construct_payload(meraki)
        have = meraki.request(meraki.construct_path("get_all", org_id=org_id), "GET")
        # meraki.fail_json(msg="Compare", have=have, payload=payload)
        if meraki.is_update_required(have, payload):
            meraki.generate_diff(have, payload)
            path = meraki.construct_path("update", org_id=org_id)
            if meraki.module.check_mode is False:
                response = meraki.request(path, "PUT", payload=json.dumps(payload))
                meraki.result["data"] = response
            else:
                meraki.result["data"] = payload
            meraki.result["changed"] = True
            meraki.exit_json(**meraki.result)
        meraki.result["data"] = have
    elif meraki.params["state"] == "absent":
        return

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr vrf_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.vrf_interfaces.vrf_interfaces import (
    Vrf_interfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.vrf_interfaces import (
    Vrf_interfacesTemplate,
)


class Vrf_interfacesFacts(object):
    """The iosxr vrf_interfaces facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Vrf_interfacesArgs.argument_spec

    def get_device_data(self, connection):
        return connection.get_config(flags="interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Vrf_interfaces network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_device_data(connection)

        # parse native config using the Vrf_interfaces template
        vrf_interfaces_parser = Vrf_interfacesTemplate(lines=data.splitlines(), module=self._module)
        objs = list(vrf_interfaces_parser.parse().values())

        ansible_facts["ansible_network_resources"].pop("vrf_interfaces", None)

        params = utils.remove_empties(
            vrf_interfaces_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["vrf_interfaces"] = params["config"]
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_live_tools_multicast_routing
short_description: Resource module for devices _live _tools _multicast _routing
description:
  - Manage operation create of the resource devices _live _tools _multicast _routing.
    - > Enqueue a job to perform a Multicast routing request for the device. This
    endpoint currently supports switches. This endpoint has a sustained rate limit
    of one request every five seconds per device, with an allowed burst of five requests.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  callback:
    description: Details for the callback. Please include either an httpServerId OR
      url and sharedSecret.
    suboptions:
      httpServer:
        description: The webhook receiver used for the callback webhook.
        suboptions:
          id:
            description: The webhook receiver ID that will receive information. If
              specifying this, please leave the url and sharedSecret fields blank.
            type: str
        type: dict
      payloadTemplate:
        description: The payload template of the webhook used for the callback.
        suboptions:
          id:
            description: The ID of the payload template. Defaults to 'wpt_00005' for
              the Callback (included) template.
            type: str
        type: dict
      sharedSecret:
        description: A shared secret that will be included in the requests sent to
          the callback URL. It can be used to verify that the request was sent by
          Meraki. If using this field, please also specify an url.
        type: str
      url:
        description: The callback URL for the webhook target. If using this field,
          please also specify a sharedSecret.
        type: str
    type: dict
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices createDeviceLiveToolsMulticastRouting
    description: Complete reference of the createDeviceLiveToolsMulticastRouting API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-live-tools-multicast-routing
notes:
  - SDK Method used are
    devices.Devices.create_device_live_tools_multicast_routing,
  - Paths used are
    post /devices/{serial}/liveTools/multicastRouting,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_live_tools_multicast_routing:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    callback:
      httpServer:
        id: aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vd2ViaG9va3M=
      payloadTemplate:
        id: wpt_2100
      sharedSecret: secret
      url: https://webhook.site/28efa24e-f830-4d9f-a12b-fbb9e5035031
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "multicastRoutingId": "string",
      "url": "string",
      "request": {
        "serial": "string"
      },
      "status": "string",
      "callback": {
        "id": "string",
        "url": "string",
        "status": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_fibre_channel_network_policy
short_description: Manage Fibre Channel Network Policies for Cisco Intersight
description:
  - Create, update, and delete Fibre Channel Network Policies on Cisco Intersight.
  - Fibre Channel Network policies configure VSAN settings for Fibre Channel virtual interfaces.
  - These policies control the default VLAN ID and VSAN ID assignments for FC networks.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/FcNetworkPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Fibre Channel Network Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Fibre Channel Network Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  default_vlan:
    description:
      - The default VLAN ID for the Fibre Channel network.
      - Valid range is 0 to 4094.
      - A value of 0 means the VLAN is not configured.
    type: int
    default: 0
  vsan_id:
    description:
      - The VSAN (Virtual Storage Area Network) ID for the Fibre Channel network.
      - Valid range is 1 to 4094.
    type: int
    default: 1
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Fibre Channel Network Policy with default settings
  cisco.intersight.intersight_fibre_channel_network_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-network-default"
    description: "Fibre Channel Network policy with default values"
    state: present

- name: Create Fibre Channel Network Policy with custom VSAN settings
  cisco.intersight.intersight_fibre_channel_network_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-network-vsan100"
    description: "Fibre Channel Network policy for VSAN 100"
    default_vlan: 100
    vsan_id: 100
    tags:
      - Key: Environment
        Value: Production
    state: present

- name: Create Fibre Channel Network Policy with VLAN and different VSAN
  cisco.intersight.intersight_fibre_channel_network_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
    name: "fc-network-custom"
    description: "Custom Fibre Channel Network policy"
    default_vlan: 200
    vsan_id: 300
    state: present

- name: Create Fibre Channel Network Policy with maximum VSAN ID
  cisco.intersight.intersight_fibre_channel_network_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-network-max-vsan"
    description: "Fibre Channel Network policy with maximum VSAN ID"
    default_vlan: 4094
    vsan_id: 4094
    state: present

- name: Update Fibre Channel Network Policy description
  cisco.intersight.intersight_fibre_channel_network_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-network-default"
    description: "Updated Fibre Channel Network policy description"
    state: present

- name: Delete Fibre Channel Network Policy
  cisco.intersight.intersight_fibre_channel_network_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-network-default"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "fc-network-vsan100",
        "ObjectType": "vnic.FcNetworkPolicy",
        "Moid": "1234567890abcdef12345678",
        "Description": "Fibre Channel Network policy for VSAN 100",
        "VsanSettings": {
            "DefaultVlanId": 100,
            "Id": 100
        },
        "Organization": {
            "Moid": "abcdef1234567890abcdef12",
            "ObjectType": "organization.Organization"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_parameters(module):
    """
    Validate module parameters for Fibre Channel Network policy configuration.
    """
    if module.params['state'] != 'present':
        return
    # Validate default_vlan range (0-4094)
    default_vlan = module.params.get('default_vlan')
    if default_vlan is not None and (default_vlan < 0 or default_vlan > 4094):
        module.fail_json(msg="Parameter 'default_vlan' must be between 0 and 4094")
    # Validate vsan_id range (1-4094)
    vsan_id = module.params.get('vsan_id')
    if vsan_id is not None and (vsan_id < 1 or vsan_id > 4094):
        module.fail_json(msg="Parameter 'vsan_id' must be between 1 and 4094")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        default_vlan=dict(type='int', default=0),
        vsan_id=dict(type='int', default=1),
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''
    # Validate module parameters
    validate_parameters(module)
    # Resource path used to configure policy
    resource_path = '/vnic/FcNetworkPolicies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name'],
    }
    if module.params['state'] == 'present':
        intersight.api_body['VsanSettings'] = {
            'DefaultVlanId': module.params['default_vlan'],
            'Id': module.params['vsan_id'],
        }
        intersight.set_tags_and_description()
    intersight.configure_policy_or_profile(resource_path=resource_path)
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2018, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: cloud_init_data_facts
short_description: Retrieve facts of cloud-init
description:
  - Gathers facts by reading the C(status.json) and C(result.json) of cloud-init.
author: René Moser (@resmo)
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.facts
  - community.general._attributes.facts_module
options:
  filter:
    description:
      - Filter facts.
    type: str
    choices: [status, result]
notes:
  - See http://cloudinit.readthedocs.io/ for more information about cloud-init.
"""

EXAMPLES = r"""
- name: Gather all facts of cloud init
  community.general.cloud_init_data_facts:
  register: result

- ansible.builtin.debug:
    var: result

- name: Wait for cloud init to finish
  community.general.cloud_init_data_facts:
    filter: status
  register: res
  until: "res.cloud_init_data_facts.status.v1.stage is defined and not res.cloud_init_data_facts.status.v1.stage"
  retries: 50
  delay: 5
"""

RETURN = r"""
ansible_facts:
  description: The returned facts.
  returned: success
  type: dict
  contains:
    cloud_init_data_facts:
      description: Facts of result and status.
      returned: success
      type: dict
      sample:
        {
          "status": {
            "v1": {
              "datasource": "DataSourceCloudStack",
              "errors": []
            }
          },
          "result": {
            "v1": {
              "datasource": "DataSourceCloudStack",
              "init": {
                "errors": [],
                "finished": 1522066377.0185432,
                "start": 1522066375.2648022
              },
              "init-local": {
                "errors": [],
                "finished": 1522066373.70919,
                "start": 1522066373.4726632
              },
              "modules-config": {
                "errors": [],
                "finished": 1522066380.9097016,
                "start": 1522066379.0011985
              },
              "modules-final": {
                "errors": [],
                "finished": 1522066383.56594,
                "start": 1522066382.3449218
              },
              "stage": null
            }
          }
        }
"""

import os

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text

CLOUD_INIT_PATH = "/var/lib/cloud/data"


def gather_cloud_init_data_facts(module):
    res = {"cloud_init_data_facts": dict()}

    for i in ["result", "status"]:
        filter = module.params.get("filter")
        if filter is None or filter == i:
            res["cloud_init_data_facts"][i] = dict()
            json_file = os.path.join(CLOUD_INIT_PATH, f"{i}.json")

            if os.path.exists(json_file):
                with open(json_file, "rb") as f:
                    contents = to_text(f.read(), errors="surrogate_or_strict")

                if contents:
                    res["cloud_init_data_facts"][i] = module.from_json(contents)
    return res


def main():
    module = AnsibleModule(
        argument_spec=dict(
            filter=dict(choices=["result", "status"]),
        ),
        supports_check_mode=True,
    )

    facts = gather_cloud_init_data_facts(module)
    result = dict(changed=False, ansible_facts=facts, **facts)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: wait_for_txt
short_description: Wait for TXT entries to be available on all authoritative nameservers
version_added: 0.1.0
description:
  - Wait for TXT entries with specific values to show up on B(all) authoritative nameservers for the DNS name.
extends_documentation_fragment:
  - community.dns._attributes
  - community.dns._attributes.idempotent_not_modify_state
attributes:
  check_mode:
    support: full
    details:
      - This action does not modify state.
    version_added: 2.4.0
  diff_mode:
    support: N/A
    details:
      - This action does not modify state.
author:
  - Felix Fontein (@felixfontein)
options:
  records:
    description:
      - A list of DNS names with TXT entries to look out for.
    required: true
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - A DNS name, like V(www.example.com).
        type: str
        required: true
      values:
        description:
          - The TXT values to look for.
          - The alias O(records[].entries) has been added in community.dns 3.4.0.
        aliases:
          - entries
        type: list
        elements: str
        required: true
      mode:
        description:
          - Comparison modes for the values in O(records[].values).
          - If V(subset), O(records[].values) should be a (not necessarily proper) subset of the TXT values set for the DNS
            name.
          - If V(superset), O(records[].values) should be a (not necessarily proper) superset of the TXT values set for the
            DNS name. This includes the case that no TXT entries are set.
          - If V(superset_not_empty), O(records[].values) should be a (not necessarily proper) superset of the TXT values
            set for the DNS name, assuming at least one TXT record is present.
          - If V(equals), O(records[].values) should be the same set of strings as the TXT values for the DNS name (up to
            order).
          - If V(equals_ordered), O(records[].values) should be the same ordered list of strings as the TXT values for the
            DNS name.
        type: str
        default: subset
        choices:
          - subset
          - superset
          - superset_not_empty
          - equals
          - equals_ordered
  query_retry:
    description:
      - Number of retries for DNS query timeouts.
    type: int
    default: 3
  query_timeout:
    description:
      - Timeout per DNS query in seconds.
    type: float
    default: 10
  timeout:
    description:
      - Global timeout for waiting for all records in seconds.
      - If not set, will wait indefinitely.
    type: float
  max_sleep:
    description:
      - Maximal amount of seconds to sleep between two rounds of probing the TXT records.
    type: float
    default: 10
  always_ask_default_resolver:
    description:
      - When set to V(true) (default), will use the default resolver to find the authoritative nameservers of a subzone. See
        O(server) for how to configure the default resolver.
      - When set to V(false), will use the authoritative nameservers of the parent zone to find the authoritative nameservers
        of a subzone. This only makes sense when the nameservers were recently changed and have not yet propagated.
    type: bool
    default: true
  servfail_retries:
    description:
      - How often to retry on SERVFAIL errors.
    type: int
    default: 0
    version_added: 2.6.0
  server:
    description:
      - The DNS server(s) to use to look up the result. Must be a list of one or more IP addresses.
      - By default, the system's standard resolver is used.
    type: list
    elements: str
    version_added: 2.7.0
requirements:
  - dnspython >= 2.0.0
"""

EXAMPLES = r"""
- name: Wait for a TXT entry to appear
  community.dns.wait_for_txt:
    records:
      # We want that www.example.com has a single TXT record with value 'Hello world!'.
      # There should not be any other TXT record for www.example.com.
      - name: www.example.com
        values: "Hello world!"
        mode: equals
      # We want that example.com has a specific SPF record set.
      # We do not care about other TXT records.
      - name: www.example.com
        values: "v=spf1 a mx -all"
        mode: subset
"""

RETURN = r"""
records:
  description:
    - Results on the TXT records queried.
    - The entries are in a 1:1 correspondence to the entries of the O(records) parameter, in exactly the same order.
  returned: always
  type: list
  elements: dict
  contains:
    name:
      description:
        - The DNS name this check is for.
      returned: always
      type: str
      sample: example.com
    done:
      description:
        - Whether the check completed.
      returned: always
      type: bool
      sample: false
    values:
      description:
        - For every authoritative nameserver for the DNS name, lists the TXT records retrieved during the last lookup made.
        - Once the check completed for all TXT records retrieved, the TXT records for this DNS name are no longer checked.
        - If these are multiple TXT entries for a nameserver, the order is as it was received from that nameserver. This might
          not be the same order provided in the check.
        - B(The field has been renamed) to RV(records[].entries) in community.dns 3.4.0.
          While the old name will be around for a longer time, prefer using the new one.
      returned: lookup was done at least once
      type: dict
      elements: list
    entries:
      description:
        - For every authoritative nameserver for the DNS name, lists the TXT records retrieved during the last lookup made.
        - Once the check completed for all TXT records retrieved, the TXT records for this DNS name are no longer checked.
        - If these are multiple TXT entries for a nameserver, the order is as it was received from that nameserver. This might
          not be the same order provided in the check.
        - This field has been called RV(records[].values) before.
      returned: lookup was done at least once
      type: dict
      elements: list
      sample:
        ns1.example.com:
          - TXT value 1
          - TXT value 2
        ns2.example.com:
          - TXT value 2
      version_added: 3.4.0
    check_count:
      description:
        - How often the TXT records for this DNS name were checked.
      returned: always
      type: int
      sample: 3
  sample:
    - name: example.com
      done: true
      entries: [a, b, c]
      check_count: 1
    - name: foo.example.org
      done: false
      check_count: 0
completed:
  description:
    - How many of the checks were completed.
  returned: always
  type: int
  sample: 3
"""

import time
import typing as t
from time import monotonic

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.dns.plugins.module_utils._resolver import (
    ResolveDirectlyFromNameServers,
    assert_requirements_present,
    guarded_run,
)

try:
    import dns.rdatatype
except ImportError:
    pass  # handled in assert_requirements_present()

if t.TYPE_CHECKING:
    import dns.rdtypes.ANY.TXT  # pragma: no cover


def lookup(resolver: ResolveDirectlyFromNameServers, name: str) -> dict[str, list[str]]:
    result = {}
    txts = resolver.resolve(name, rdtype=dns.rdatatype.TXT)
    for key, txt in txts.items():
        res = []
        if txt is not None:
            for data in txt:
                line = []
                for txtstring in data.strings:
                    line.append(to_text(txtstring))
                res.append("".join(line))
        result[key] = res
    return result


def validate_check(
    record_values: list[str],
    expected_values: list[str],
    comparison_mode: t.Literal[
        "subset", "superset", "superset_not_empty", "equals", "equals_ordered"
    ],
) -> bool:
    if comparison_mode == "subset":
        return set(expected_values) <= set(record_values)

    if comparison_mode == "superset":
        return set(expected_values) >= set(record_values)

    if comparison_mode == "superset_not_empty":
        return bool(record_values) and set(expected_values) >= set(record_values)

    if comparison_mode == "equals":
        return sorted(record_values) == sorted(expected_values)

    if comparison_mode == "equals_ordered":
        return record_values == expected_values

    raise AssertionError("Internal error!")  # pragma: no cover


class Waiter:
    def __init__(self, module: AnsibleModule) -> None:
        self.module = module

        self.resolver = ResolveDirectlyFromNameServers(
            timeout=self.module.params["query_timeout"],
            timeout_retries=self.module.params["query_retry"],
            servfail_retries=self.module.params["servfail_retries"],
            always_ask_default_resolver=self.module.params[
                "always_ask_default_resolver"
            ],
            server_addresses=self.module.params["server"],
        )
        self.records: list[dict[str, t.Any]] = self.module.params["records"]
        self.timeout: float | None = self.module.params["timeout"]
        self.max_sleep: float = self.module.params["max_sleep"]

        self.results = [
            {
                "name": record["name"],
                "done": False,
                "check_count": 0,
            }
            for record in self.records
        ]
        self.finished_checks = 0

    def _run(self) -> None:
        start_time = monotonic()

        step = 0
        while True:
            has_timeout = False
            if self.timeout is not None:
                expired = monotonic() - start_time
                has_timeout = expired > self.timeout

            done = True
            for index, record in enumerate(self.records):
                if self.results[index]["done"]:
                    continue
                txts = lookup(self.resolver, record["name"])
                self.results[index]["values"] = txts
                self.results[index]["entries"] = txts
                self.results[index]["check_count"] += 1
                if txts and all(
                    validate_check(txt, record["values"], record["mode"])
                    for txt in txts.values()
                ):
                    self.results[index]["done"] = True
                    self.finished_checks += 1
                else:
                    done = False

            if done:
                self.module.exit_json(
                    msg="All checks passed", **self._generate_additional_results()
                )

            if has_timeout:
                self.module.fail_json(
                    msg=f"Timeout ({self.finished_checks} out of {len(self.records)} check(s) passed).",
                    **self._generate_additional_results(),
                )

            # Simple quadratic sleep with maximum wait of max_sleep seconds
            wait = min(2 + step * 0.5, self.max_sleep)
            if self.timeout is not None:
                # Make sure we do not exceed the timeout by much by waiting
                expired = monotonic() - start_time
                wait = max(min(wait, self.timeout - expired + 0.1), 0.1)

            time.sleep(wait)
            step += 1

    def _generate_additional_results(self) -> dict[str, t.Any]:
        return {
            "records": self.results,
            "completed": self.finished_checks,
        }

    def run(self) -> None:
        guarded_run(
            self._run,
            self.module,
            generate_additional_results=self._generate_additional_results,
        )


def main() -> None:
    module = AnsibleModule(
        argument_spec={
            "records": {
                "required": True,
                "type": "list",
                "elements": "dict",
                "options": {
                    "name": {"required": True, "type": "str"},
                    "values": {
                        "required": True,
                        "type": "list",
                        "elements": "str",
                        "aliases": ["entries"],
                    },
                    "mode": {
                        "type": "str",
                        "default": "subset",
                        "choices": [
                            "subset",
                            "superset",
                            "superset_not_empty",
                            "equals",
                            "equals_ordered",
                        ],
                    },
                },
            },
            "query_retry": {"type": "int", "default": 3},
            "query_timeout": {"type": "float", "default": 10},
            "timeout": {"type": "float"},
            "max_sleep": {"type": "float", "default": 10},
            "always_ask_default_resolver": {"type": "bool", "default": True},
            "servfail_retries": {"type": "int", "default": 0},
            "server": {"type": "list", "elements": "str"},
        },
        supports_check_mode=True,
    )
    assert_requirements_present(module)

    waiter = Waiter(module)
    waiter.run()


if __name__ == "__main__":
    main()



# Copyright (c) 2018, Huawei Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # HWC doc fragment.
    DOCUMENTATION = r"""
options:
  identity_endpoint:
    description:
      - The Identity authentication URL.
    type: str
    required: true
  user:
    description:
      - The user name to login with.
      - Currently only user names are supported, and not user IDs.
    type: str
    required: true
  password:
    description:
      - The password to login with.
    type: str
    required: true
  domain:
    description:
      - The name of the Domain to scope to (Identity v3).
      - Currently only domain names are supported, and not domain IDs.
    type: str
    required: true
  project:
    description:
      - The name of the Tenant (Identity v2) or Project (Identity v3).
      - Currently only project names are supported, and not project IDs.
    type: str
    required: true
  region:
    description:
      - The region to which the project belongs.
    type: str
  id:
    description:
      - The ID of resource to be managed.
    type: str
notes:
  - For authentication, you can set identity_endpoint using the E(ANSIBLE_HWC_IDENTITY_ENDPOINT) environment variable.
  - For authentication, you can set user using the E(ANSIBLE_HWC_USER) environment variable.
  - For authentication, you can set password using the E(ANSIBLE_HWC_PASSWORD) environment variable.
  - For authentication, you can set domain using the E(ANSIBLE_HWC_DOMAIN) environment variable.
  - For authentication, you can set project using the E(ANSIBLE_HWC_PROJECT) environment variable.
  - For authentication, you can set region using the E(ANSIBLE_HWC_REGION) environment variable.
  - Environment variables values are only used when the playbook values are not set.
"""



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitordatacollectionruleassociation
version_added: "3.9.0"
short_description: Managed Data Collection Rule Association
description:
    - Create, update or delete the Data Collection Rule Association.

options:
    data_collection_endpoint_id:
        description:
            - The ID of the data collection endpoint.
        type: str
    data_collection_rule_id:
        description:
            - The ID of the data collection rule.
        type: str
    resource_uri:
        description:
            - The identifier of the resource.
        type: str
        required: true
    association_name:
        description:
            - The name of the association.
            - The name is case insensitive.
            - An association of data collection endpoint must be named 'configurationAccessEndpoint.
        type: str
        required: true
    description:
        description:
            - Description of the association.
        type: str
    state:
        description:
            - State of the data colleciton rule association.
            - Set to C(present) to create a new association.
            - Set to C(absent) to remove a association.
        default: present
        type: str
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a new data collection rule association
  azure.azcollection.azure_rm_monitordatacollectionrulesassociation:
    resource_uri: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/fredVM"
    association_name: association01
    data_collection_rule_id: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG02/providers/Microsoft.Insights/dataCollectionRules/fredrpfx001-DCR"
    description: fredtest

- name: Delete the data collection rule association
  azure.azcollection.azure_rm_monitordatacollectionrulesassociation:
    resource_uri: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/fredVM"
    association_name: association01
    state: absent
'''

RETURN = '''
datacollectionruleassociation:
    description:
        - The facts of the data collection rule association.
    type: complex
    returned: always
    contains:
        data_collection_rule_id:
            description:
                - The resource ID of the data collection endpoint that is to be associated.
            type: str
            returned: when-used
            sample: "/subscriptions/xxxxx/resourceGroups/v-xisuRG02/providers/Microsoft.Insights/dataCollectionRules/fredrpfx001-DCR"
        data_collection_endpoint_id:
            description:
                - The resource ID of the data collection endpoint that is to be associated.
            type: str
            returned: when-used
            sample: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Insights/dataCollectionEndpoints/fredendpoint"
        description:
            description:
                - Description of the association.
            type: str
            returned: always
            sample: Create
        etag:
            description:
                - Description of the association.
            returned: always
            type: str
            sample: "0c00c2b5-0000-0800-0000-68ca0de80000"
        id:
            description:
                - Fully qualified ID of the resource.
            type: str
            returned: always
            sample: "/subscriptions/xxxxxxxxxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/\
                     fredVM/providers/Microsoft.Insights/dataCollectionRuleAssociations/association01"
        name:
            description:
                - Description of the association.
            type: str
            returned: always
            sample: associationname
        system_data:
            description:
                - Metadata pertaining to creation and last modification of the resource.
            type: dict
            returned: always
            sample: {
                "created_at": "2025-09-17T01:24:56.450551Z",
                "created_by": "00867800-0fa3-4d02-8bc8-35edac3a0d32",
                "created_by_type": "Application",
                "last_modified_at": "2025-09-17T01:24:56.450551Z",
                "last_modified_by": "00867800-0fa3-4d02-8bc8-35edac3a0d32",
                "last_modified_by_type": "Application"
            }
        type:
            description:
                - The type of the resource.
            type: str
            returned: always
            sample: "Microsoft.Insights/dataCollectionRuleAssociations"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMDataCollectionRuleAssociation(AzureRMModuleBase):
    """Information class for an Azure RM Data Collection Rules"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_uri=dict(type='str', required=True),
            association_name=dict(type='str', required=True),
            data_collection_endpoint_id=dict(type='str'),
            data_collection_rule_id=dict(type='str'),
            description=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent'])
        )

        self.resource_uri = None
        self.association_name = None
        self.data_collection_endpoint_id = None
        self.data_collection_rule_id = None
        self.state = None
        self.description = None
        self.log_path = None
        self.log_mode = None
        mutually_exclusive = [('data_collection_endpoint_id', 'data_collection_rule_id')]

        self.results = dict(
            changed=False,
            datacollectionruleassociation=None
        )

        super(AzureRMDataCollectionRuleAssociation, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                   supports_check_mode=True,
                                                                   supports_tags=False,
                                                                   mutually_exclusive=mutually_exclusive,
                                                                   facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        response = self.get_association()
        changed = False
        if self.state == 'present':
            if response:
                if self.description and self.description != response.get('description'):
                    changed = True
                else:
                    self.description = response.get('description')
                if self.data_collection_rule_id and self.data_collection_rule_id != response.get('data_collection_rule_id'):
                    self.fail("Already exist assciation with {0}. If need to udpate, please delete it and recreate it".format(self.data_collection_rule_id))
                else:
                    self.data_collection_rule_id = response.get('data_collection_rule_id')
                if self.data_collection_endpoint_id and self.data_collection_endpoint_id != response.get('data_collection_endpoint_id'):
                    self.fail("Already exist associat with {0}. If need to udpate, please delete it and recreate it".format(self.data_collection_endpoint_id))
                else:
                    self.data_collection_endpoint_id = response.get('data_collection_endpoint_id')
            else:
                changed = True

            if not self.check_mode and changed:
                response = self.create_association()
        else:
            if response:
                changed = True
                if self.check_mode:
                    self.log("The monitor data collection rule association already exist, will be delete")
                else:
                    response = self.delete_association()
            else:
                if self.check_mode:
                    self.log("There is no monitor data collection rule association.")

        self.results['datacollectionruleassociation'] = response
        self.results['changed'] = changed

        return self.results

    def get_association(self):
        '''
        Gets the specified association
        '''
        response = None
        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_rule_associations.get(resource_uri=self.resource_uri,
                                                                                                                  association_name=self.association_name)
        except Exception:
            self.log("Could not find data collection rule assoication {0} in resource uri {1}".format(self.association_name, self.resource_uri))
        if response:
            return response.as_dict()

    def create_association(self):
        '''
        Creates or updates an association.
        '''
        response = None
        try:
            body = dict(description=self.description,
                        data_collection_rule_id=self.data_collection_rule_id,
                        data_collection_endpoint_id=self.data_collection_endpoint_id)
            response = self.monitor_management_client_data_collection_rules.data_collection_rule_associations.create(resource_uri=self.resource_uri,
                                                                                                                     association_name=self.association_name,
                                                                                                                     body=body)
        except Exception as ex:
            self.fail("Creates or update the association occurred exception, Exception as {0}".format(ex))
        return response.as_dict()

    def delete_association(self):
        '''
        Deletes an association
        '''
        try:
            self.monitor_management_client_data_collection_rules.data_collection_rule_associations.delete(resource_uri=self.resource_uri,
                                                                                                          association_name=self.association_name)
        except Exception as ex:
            self.fail("Deletes the association occurred exception, Exception as {0}".format(ex))


def main():
    """Main execution"""
    AzureRMDataCollectionRuleAssociation()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2024, Ryan Cook <rcook@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt
# or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: bootc_manage
version_added: 9.3.0
author:
  - Ryan Cook (@cooktheryan)
short_description: Bootc Switch and Upgrade
description:
  - This module manages the switching and upgrading of C(bootc).
options:
  state:
    description:
      - Control whether to apply the latest image or switch the image.
      - B(Note:) This does not reboot the system.
      - Please use M(ansible.builtin.reboot) to reboot the system.
    required: true
    type: str
    choices: ['switch', 'latest']
  image:
    description:
      - The image to switch to.
      - This is required when O(state=switch).
    type: str
"""

EXAMPLES = r"""
# Switch to a different image
- name: Provide image to switch to a different image and retain the current running image
  community.general.bootc_manage:
    state: switch
    image: "example.com/image:latest"

# Apply updates of the current running image
- name: Apply updates of the current running image
  community.general.bootc_manage:
    state: latest
"""

RETURN = r"""
"""


from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.locale import get_best_parsable_locale


def main():
    argument_spec = dict(
        state=dict(type="str", required=True, choices=["switch", "latest"]),
        image=dict(type="str"),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=[
            ("state", "switch", ["image"]),
        ],
    )

    state = module.params["state"]
    image = module.params["image"]

    if state == "switch":
        command = ["bootc", "switch", image, "--retain"]
    elif state == "latest":
        command = ["bootc", "upgrade"]

    locale = get_best_parsable_locale(module)
    module.run_command_environ_update = dict(
        LANG=locale, LC_ALL=locale, LC_MESSAGES=locale, LC_CTYPE=locale, LANGUAGE=locale
    )
    rc, stdout, err = module.run_command(command, check_rc=True)

    if "Queued for next boot: " in stdout:
        result = {"changed": True, "stdout": stdout}
        module.exit_json(**result)
    elif "No changes in " in stdout or "Image specification is unchanged." in stdout:
        result = {"changed": False, "stdout": stdout}
        module.exit_json(**result)
    else:
        result = {"changed": False, "stderr": err}
        module.fail_json(msg="ERROR: Command execution failed.", **result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2025 Klaas Demter (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_recoveryservicesvaultconfig_info
version_added: "3.4.0"
short_description: Get Info on Azure Backup Vault Configs
description:
    - Get Azure Backup Vault Config
    - Most of the information here is also part of azure_rm_recoveryservicesvault_info
    - it's seperated out because of https://github.com/Azure/azure-rest-api-specs/issues/34218

options:
    vault_name:
        description:
            - The name of the Recovery Services Vault the vault config belongs to.
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group the vault is in.
        required: true
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Klaas Demter (@Klaas-)
'''

EXAMPLES = '''
- name: Get backup vault config information
  azure.azcollection.azure_rm_recoveryservicesvaultconfig_info:
    vault_name: Vault_Name
    resource_group: Resource_Group_Name
'''

RETURN = '''
id:
    description:
        - Id of specified backup vault config.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/yyyyyyyyy/
            providers/Microsoft.RecoveryServices/vaults/zzzzzzzzzz/backupconfig/vaultconfig"
name:
    description:
        - always returns "vaultconfig"
    type: str
    returned: always
    sample: "vaultconfig"
type:
    description:
        - Type of backup vault config.
    type: str
    returned: always
    sample: "Microsoft.RecoveryServices/vaults/backupconfig"
properties:
    description:
        - Attributes of the backup vault config.
    type: dict
    returned: always
    sample: {
            "enhanced_security_state": "Enabled",
            "is_soft_delete_feature_state_editable": true,
            "soft_delete_feature_state": "Enabled",
            "soft_delete_retention_period_in_days": 14
        }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError

except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMBackupVaultConfigInfo(AzureRMModuleBase):
    """Information class for an Azure RM Backup Vault Configs"""

    def __init__(self):
        self.module_arg_spec = dict(
            vault_name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
        )

        self.vault_name = None
        self.resource_group = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            id=None,
            changed=False
        )

        super(AzureRMBackupVaultConfigInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                           supports_check_mode=True,
                                                           supports_tags=False,
                                                           facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        existing_backup_vault_config = None
        response = None

        existing_backup_vault_config = self.get_vault_config()

        self.set_results(existing_backup_vault_config)

        return self.results

    def get_vault_config(self):
        '''
        Gets the properties of the specified backup vault config.
        https://learn.microsoft.com/en-us/python/api/azure-mgmt-recoveryservicesbackup/azure.mgmt.recoveryservicesbackup.activestamp.aio.operations.backupresourcevaultconfigsoperations?view=azure-python

        :return: BackupResourceVaultConfigResource
        '''
        self.log("Checking for vault config of vault {0} in resource group {1}".format(self.vault_name, self.resource_group))

        vault_config = None

        try:
            vault_config = self.recovery_services_backup_client.backup_resource_vault_configs.get(vault_name=self.vault_name,
                                                                                                  resource_group_name=self.resource_group)
        except ResourceNotFoundError as ex:
            self.log("Could not find vault {0} in resource group {1}".format(self.vault_name, self.resource_group))

        return vault_config

    def set_results(self, vault_config):
        if vault_config:
            self.results['id'] = vault_config.id
            # Not properly returned by sdk nor API
            # https://github.com/Azure/azure-rest-api-specs/issues/34530#issuecomment-2862950321
            # self.results['location'] = vault_config.location
            self.results['name'] = vault_config.name
            self.results['type'] = vault_config.type
            self.results['properties'] = vault_config.properties.as_dict()

        else:
            self.results['id'] = None
            self.results['name'] = None
            self.results['type'] = None


def main():
    """Main execution"""
    AzureRMBackupVaultConfigInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2021, Alexei Znamensky <russoz@gmail.com>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ansible_galaxy_install
author:
  - "Alexei Znamensky (@russoz)"
short_description: Install Ansible roles or collections using ansible-galaxy
version_added: 3.5.0
description:
  - This module allows the installation of Ansible collections or roles using C(ansible-galaxy).
notes:
  - Support for B(Ansible 2.9/2.10) was removed in community.general 8.0.0.
  - The module tries to run using the C(C.UTF-8) locale. If that fails, it tries C(en_US.UTF-8). If that one also fails, the
    module fails.
seealso:
  - name: C(ansible-galaxy) command manual page
    description: Manual page for the command.
    link: https://docs.ansible.com/projects/ansible/latest/cli/ansible-galaxy.html

requirements:
  - ansible-core 2.11 or newer
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    description:
      - If O(state=present) then the collection or role is installed. Note that the collections and roles are not updated
        with this option.
      - Currently the O(state=latest) is ignored unless O(type=collection), and it ensures the collection is installed and
        updated to the latest available version.
      - Please note that O(force=true) can be used to perform upgrade regardless of O(type).
    type: str
    choices: [present, latest]
    default: present
    version_added: 9.1.0
  type:
    description:
      - The type of installation performed by C(ansible-galaxy).
      - If O(type=both), then O(requirements_file) must be passed and it may contain both roles and collections.
      - 'Note however that the opposite is not true: if using a O(requirements_file), then O(type) can be any of the three
        choices.'
    type: str
    choices: [collection, role, both]
    required: true
  name:
    description:
      - Name of the collection or role being installed.
      - Versions can be specified with C(ansible-galaxy) usual formats. For example, the collection V(community.docker:1.6.1)
        or the role V(ansistrano.deploy,3.8.0).
      - O(name) and O(requirements_file) are mutually exclusive.
    type: str
  requirements_file:
    description:
      - Path to a file containing a list of requirements to be installed.
      - It works for O(type) equals to V(collection) and V(role).
      - O(name) and O(requirements_file) are mutually exclusive.
    type: path
  dest:
    description:
      - The path to the directory containing your collections or roles, according to the value of O(type).
      - Please notice that C(ansible-galaxy) does not install collections with O(type=both), when O(requirements_file) contains
        both roles and collections and O(dest) is specified.
    type: path
  executable:
    description:
      - Path to the C(ansible-galaxy) executable.
      - When not specified, the module uses C(ansible-galaxy) found by Ansible.
    type: path
    version_added: 12.5.0
  no_deps:
    description:
      - Refrain from installing dependencies.
    version_added: 4.5.0
    type: bool
    default: false
  force:
    description:
      - Force overwriting existing roles and/or collections.
      - It can be used for upgrading, but the module output always reports RV(ignore:changed=true).
      - Using O(force=true) is mandatory when downgrading.
    type: bool
    default: false
"""

EXAMPLES = r"""
- name: Install collection community.network
  community.general.ansible_galaxy_install:
    type: collection
    name: community.network

- name: Install role at specific path
  community.general.ansible_galaxy_install:
    type: role
    name: ansistrano.deploy
    dest: /ansible/roles

- name: Install collections and roles together
  community.general.ansible_galaxy_install:
    type: both
    requirements_file: requirements.yml

- name: Force-install collection community.network at specific version
  community.general.ansible_galaxy_install:
    type: collection
    name: community.network:3.0.2
    force: true
"""

RETURN = r"""
type:
  description: The value of the O(type) parameter.
  type: str
  returned: always
name:
  description: The value of the O(name) parameter.
  type: str
  returned: always
dest:
  description: The value of the O(dest) parameter.
  type: str
  returned: always
requirements_file:
  description: The value of the O(requirements_file) parameter.
  type: str
  returned: always
force:
  description: The value of the O(force) parameter.
  type: bool
  returned: always
installed_roles:
  description:
    - If O(requirements_file) is specified instead, returns dictionary with all the roles installed per path.
    - If O(name) is specified, returns that role name and the version installed per path.
  type: dict
  returned: always when installing roles
  contains:
    "<path>":
      description: Roles and versions for that path.
      type: dict
  sample:
    /home/user42/.ansible/roles:
      ansistrano.deploy: 3.9.0
      baztian.xfce: v0.0.3
    /custom/ansible/roles:
      ansistrano.deploy: 3.8.0
installed_collections:
  description:
    - If O(requirements_file) is specified instead, returns dictionary with all the collections installed per path.
    - If O(name) is specified, returns that collection name and the version installed per path.
  type: dict
  returned: always when installing collections
  contains:
    "<path>":
      description: Collections and versions for that path.
      type: dict
  sample:
    /home/az/.ansible/collections/ansible_collections:
      community.docker: 1.6.0
      community.general: 3.0.2
    /custom/ansible/ansible_collections:
      community.general: 3.1.0
new_collections:
  description: New collections installed by this module.
  returned: success
  type: dict
  sample:
    community.general: 3.1.0
    community.docker: 1.6.1
new_roles:
  description: New roles installed by this module.
  returned: success
  type: dict
  sample:
    ansistrano.deploy: 3.8.0
    baztian.xfce: v0.0.3
version:
  description: Version of ansible-core for ansible-galaxy.
  type: str
  returned: always
  sample: 2.17.4
  version_added: 10.0.0
"""

import re

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt
from ansible_collections.community.general.plugins.module_utils._module_helper import (
    ModuleHelper,
    ModuleHelperException,
)


class AnsibleGalaxyInstall(ModuleHelper):
    _RE_GALAXY_VERSION = re.compile(r"^ansible-galaxy(?: \[core)? (?P<version>\d+\.\d+\.\d+)(?:\.\w+)?(?:\])?")
    _RE_LIST_PATH = re.compile(r"^# (?P<path>.*)$")
    _RE_LIST_COLL = re.compile(r"^(?P<elem>\w+\.\w+)\s+(?P<version>[\d\.]+)\s*$")
    _RE_LIST_ROLE = re.compile(r"^- (?P<elem>\w+\.\w+),\s+(?P<version>[\d\.]+)\s*$")
    _RE_INSTALL_OUTPUT = re.compile(
        r"^(?:(?P<collection>\w+\.\w+):(?P<cversion>[\d\.]+)|- (?P<role>\w+\.\w+) \((?P<rversion>[\d\.]+)\)) was installed successfully$"
    )
    ansible_version = None

    output_params = ("type", "name", "dest", "requirements_file", "executable", "force", "no_deps")
    module = dict(
        argument_spec=dict(
            state=dict(type="str", choices=["present", "latest"], default="present"),
            type=dict(type="str", choices=("collection", "role", "both"), required=True),
            name=dict(type="str"),
            requirements_file=dict(type="path"),
            dest=dict(type="path"),
            executable=dict(type="path"),
            force=dict(type="bool", default=False),
            no_deps=dict(type="bool", default=False),
        ),
        mutually_exclusive=[("name", "requirements_file")],
        required_one_of=[("name", "requirements_file")],
        required_if=[("type", "both", ["requirements_file"])],
        supports_check_mode=False,
    )

    command = "ansible-galaxy"
    command_args_formats = dict(
        type=cmd_runner_fmt.as_func(lambda v: [] if v == "both" else [v]),
        galaxy_cmd=cmd_runner_fmt.as_list(),
        upgrade=cmd_runner_fmt.as_bool("--upgrade"),
        requirements_file=cmd_runner_fmt.as_opt_val("-r"),
        dest=cmd_runner_fmt.as_opt_val("-p"),
        force=cmd_runner_fmt.as_bool("--force"),
        no_deps=cmd_runner_fmt.as_bool("--no-deps"),
        version=cmd_runner_fmt.as_fixed("--version"),
        name=cmd_runner_fmt.as_list(),
    )

    def _make_runner(self, lang):
        return CmdRunner(
            self.module,
            command=self.vars.executable or self.command,
            arg_formats=self.command_args_formats,
            force_lang=lang,
            check_rc=True,
        )

    def _get_ansible_galaxy_version(self):
        class UnsupportedLocale(ModuleHelperException):
            pass

        def process(rc, out, err):
            if (rc != 0 and "unsupported locale setting" in err) or (rc == 0 and "cannot change locale" in err):
                raise UnsupportedLocale(msg=err)
            line = out.splitlines()[0]
            match = self._RE_GALAXY_VERSION.match(line)
            if not match:
                self.do_raise(f"Unable to determine ansible-galaxy version from: {line}")
            version = match.group("version")
            return version

        try:
            runner = self._make_runner("C.UTF-8")
            with runner("version", check_rc=False, output_process=process) as ctx:
                return runner, ctx.run()
        except UnsupportedLocale:
            runner = self._make_runner("en_US.UTF-8")
            with runner("version", check_rc=True, output_process=process) as ctx:
                return runner, ctx.run()

    def __init_module__(self):
        self.runner, self.vars.version = self._get_ansible_galaxy_version()
        self.ansible_version = tuple(int(x) for x in self.vars.version.split(".")[:3])
        if self.ansible_version < (2, 11):
            self.do_raise(msg="Support for Ansible 2.9 and ansible-base 2.10 has been removed.")
        self.vars.set("new_collections", {}, change=True)
        self.vars.set("new_roles", {}, change=True)
        if self.vars.type != "collection":
            self.vars.installed_roles = self._list_roles()
        if self.vars.type != "roles":
            self.vars.installed_collections = self._list_collections()

    def _list_element(self, _type, path_re, elem_re):
        def process(rc, out, err):
            return [] if "None of the provided paths were usable" in out else out.splitlines()

        with self.runner("type galaxy_cmd dest", output_process=process, check_rc=False) as ctx:
            elems = ctx.run(type=_type, galaxy_cmd="list")

        elems_dict = {}
        current_path = None
        for line in elems:
            if line.startswith("#"):
                match = path_re.match(line)
                if not match:
                    continue
                if self.vars.dest is not None and match.group("path") != self.vars.dest:
                    current_path = None
                    continue
                current_path = match.group("path") if match else None
                elems_dict[current_path] = {}

            elif current_path is not None:
                match = elem_re.match(line)
                if not match or (self.vars.name is not None and match.group("elem") != self.vars.name):
                    continue
                elems_dict[current_path][match.group("elem")] = match.group("version")
        return elems_dict

    def _list_collections(self):
        return self._list_element("collection", self._RE_LIST_PATH, self._RE_LIST_COLL)

    def _list_roles(self):
        return self._list_element("role", self._RE_LIST_PATH, self._RE_LIST_ROLE)

    def __run__(self):
        def process(rc, out, err):
            for line in out.splitlines():
                match = self._RE_INSTALL_OUTPUT.match(line)
                if not match:
                    continue
                if match.group("collection"):
                    self.vars.new_collections[match.group("collection")] = match.group("cversion")
                elif match.group("role"):
                    self.vars.new_roles[match.group("role")] = match.group("rversion")

        upgrade = self.vars.type == "collection" and self.vars.state == "latest"
        with self.runner(
            "type galaxy_cmd upgrade force no_deps dest requirements_file name", output_process=process
        ) as ctx:
            ctx.run(galaxy_cmd="install", upgrade=upgrade)
            if self.verbosity > 2:
                self.vars.set("run_info", ctx.run_info)


def main():
    AnsibleGalaxyInstall.execute()


if __name__ == "__main__":
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Luke Murphy @decentral1se
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

from ansible.module_utils.ansible_release import __version__ as ansible_version


def get_user_agent(module: str) -> str:
    """Retrieve a user-agent to send with LinodeClient requests."""
    return f"Ansible-{module}/{ansible_version}"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---

version_added: 1.0.0
module: cloudfront_invalidation

short_description: create invalidations for AWS CloudFront distributions
description:
  - Allows for invalidation of a batch of paths for a CloudFront distribution.

author:
  - Willem van Ketwich (@wilvk)

options:
    distribution_id:
      description:
        - The ID of the CloudFront distribution to invalidate paths for. Can be specified instead of the alias.
      required: false
      type: str
    alias:
      description:
        - The alias of the CloudFront distribution to invalidate paths for. Can be specified instead of distribution_id.
      required: false
      type: str
    caller_reference:
      description:
        - A unique reference identifier for the invalidation paths.
        - Defaults to current datetime stamp.
      required: false
      default:
      type: str
    target_paths:
      description:
        - A list of paths on the distribution to invalidate. Each path should begin with C(/). Wildcards are allowed. eg. C(/foo/bar/*)
      required: true
      type: list
      elements: str

notes:
  - does not support check mode

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: create a batch of invalidations using a distribution_id for a reference
  community.aws.cloudfront_invalidation:
    distribution_id: E15BU8SDCGSG57
    caller_reference: testing 123
    target_paths:
      - /testpathone/test1.css
      - /testpathtwo/test2.js
      - /testpaththree/test3.ss

- name: create a batch of invalidations using an alias as a reference and one path using a wildcard match
  community.aws.cloudfront_invalidation:
    alias: alias.test.com
    caller_reference: testing 123
    target_paths:
      - /testpathone/test4.css
      - /testpathtwo/test5.js
      - /testpaththree/*
"""

RETURN = r"""
invalidation:
  description: The invalidation's information.
  returned: always
  type: complex
  contains:
    create_time:
      description: The date and time the invalidation request was first made.
      returned: always
      type: str
      sample: '2018-02-01T15:50:41.159000+00:00'
    id:
      description: The identifier for the invalidation request.
      returned: always
      type: str
      sample: I2G9MOWJZFV612
    invalidation_batch:
      description: The current invalidation information for the batch request.
      returned: always
      type: complex
      contains:
        caller_reference:
          description: The value used to uniquely identify an invalidation request.
          returned: always
          type: str
          sample: testing 123
        paths:
          description: A dict that contains information about the objects that you want to invalidate.
          returned: always
          type: complex
          contains:
            items:
              description:
                - A list of the paths that you want to invalidate.
                - This return value has been deprecated and will be removed in a release after 2026-12-16.
                  Use RV(invalidation.invalidation_batch.paths.elements) instead.
              returned: always
              type: list
              sample:
              - /testpathtwo/test2.js
              - /testpathone/test1.css
              - /testpaththree/test3.ss
            elements:
              description:
                - A list of the paths that you want to invalidate.
              returned: always
              type: list
              sample:
              - /testpathtwo/test2.js
              - /testpathone/test1.css
              - /testpaththree/test3.ss
              version_added: 10.1.0
            quantity:
              description: The number of objects that you want to invalidate.
              returned: always
              type: int
              sample: 3
    status:
      description: The status of the invalidation request.
      returned: always
      type: str
      sample: Completed
location:
  description: The fully qualified URI of the distribution and invalidation batch request.
  returned: always
  type: str
  sample: https://cloudfront.amazonaws.com/2017-03-25/distribution/E1ZID6KZJECZY7/invalidation/I2G9MOWJZFV622
"""

import datetime

try:
    import botocore
except ImportError:
    pass  # caught by imported AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_message
from ansible_collections.amazon.aws.plugins.module_utils.cloudfront_facts import CloudFrontFactsServiceManager

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class CloudFrontInvalidationServiceManager(object):
    """
    Handles CloudFront service calls to AWS for invalidations
    """

    def __init__(self, module, cloudfront_facts_mgr):
        self.module = module
        self.client = module.client("cloudfront")
        self.__cloudfront_facts_mgr = cloudfront_facts_mgr

    def create_invalidation(self, distribution_id, invalidation_batch):
        current_invalidation_response = self.get_invalidation(distribution_id, invalidation_batch["CallerReference"])
        try:
            response = self.client.create_invalidation(
                DistributionId=distribution_id, InvalidationBatch=invalidation_batch
            )
            response.pop("ResponseMetadata", None)
            if current_invalidation_response:
                return response, False
            else:
                return response, True
        except is_boto3_error_message(
            "Your request contains a caller reference that was used for a previous invalidation "
            "batch for the same distribution."
        ):
            self.module.warn(
                "InvalidationBatch target paths are not modifiable. "
                "To make a new invalidation please update caller_reference."
            )
            return current_invalidation_response, False
        except (
            botocore.exceptions.ClientError,
            botocore.exceptions.BotoCoreError,
        ) as e:  # pylint: disable=duplicate-except
            self.module.fail_json_aws(e, msg="Error creating CloudFront invalidations.")

    def get_invalidation(self, distribution_id, caller_reference):
        # find all invalidations for the distribution
        invalidations = self.__cloudfront_facts_mgr.list_invalidations(distribution_id=distribution_id)

        # check if there is an invalidation with the same caller reference
        for invalidation in invalidations:
            invalidation_info = self.__cloudfront_facts_mgr.get_invalidation(
                distribution_id=distribution_id, id=invalidation["Id"]
            )
            if invalidation_info.get("InvalidationBatch", {}).get("CallerReference") == caller_reference:
                invalidation_info.pop("ResponseMetadata", None)
                return invalidation_info
        return {}


class CloudFrontInvalidationValidationManager(object):
    """
    Manages CloudFront validations for invalidation batches
    """

    def __init__(self, module, cloudfront_facts_mgr):
        self.module = module
        self.__cloudfront_facts_mgr = cloudfront_facts_mgr

    def validate_distribution_id(self, distribution_id, alias):
        try:
            if distribution_id is None and alias is None:
                self.module.fail_json(msg="distribution_id or alias must be specified")
            if distribution_id is None:
                distribution_id = self.__cloudfront_facts_mgr.get_distribution_id_from_domain_name(alias)
            return distribution_id
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error validating parameters.")

    def create_aws_list(self, invalidation_batch):
        aws_list = {}
        aws_list["Quantity"] = len(invalidation_batch)
        aws_list["Items"] = invalidation_batch
        return aws_list

    def validate_invalidation_batch(self, invalidation_batch, caller_reference):
        try:
            if caller_reference is not None:
                valid_caller_reference = caller_reference
            else:
                valid_caller_reference = datetime.datetime.now().isoformat()
            valid_invalidation_batch = {
                "paths": self.create_aws_list(invalidation_batch),
                "caller_reference": valid_caller_reference,
            }
            return valid_invalidation_batch
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error validating invalidation batch.")


def main():
    argument_spec = dict(
        caller_reference=dict(),
        distribution_id=dict(),
        alias=dict(),
        target_paths=dict(required=True, type="list", elements="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec, supports_check_mode=False, mutually_exclusive=[["distribution_id", "alias"]]
    )

    cloudfront_facts_mgr = CloudFrontFactsServiceManager(module)
    validation_mgr = CloudFrontInvalidationValidationManager(module, cloudfront_facts_mgr)
    service_mgr = CloudFrontInvalidationServiceManager(module, cloudfront_facts_mgr)

    caller_reference = module.params.get("caller_reference")
    distribution_id = module.params.get("distribution_id")
    alias = module.params.get("alias")
    target_paths = module.params.get("target_paths")

    result = {}

    distribution_id = validation_mgr.validate_distribution_id(distribution_id, alias)
    valid_target_paths = validation_mgr.validate_invalidation_batch(target_paths, caller_reference)
    valid_pascal_target_paths = snake_dict_to_camel_dict(valid_target_paths, True)
    result, changed = service_mgr.create_invalidation(distribution_id, valid_pascal_target_paths)

    # Duplicate 'Invalidation.InvalidationBatch.Paths.Items' to 'Invalidation.InvalidationBatch.Paths.Elements'
    result["Invalidation"]["InvalidationBatch"]["Paths"]["Elements"] = result["Invalidation"]["InvalidationBatch"][
        "Paths"
    ]["Items"]

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(result))


if __name__ == "__main__":
    main()
